@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hci_lib.h,v 1.41 2005/01/29 03:24:48 holtmann Exp $
+ *  $Id: hci_lib.h,v 1.51 2005/09/07 16:14:55 holtmann Exp $
  */
 
 #ifndef __HCI_LIB_H
@@ -69,11 +69,16 @@ int hci_devid(const char *str);
 int hci_read_local_name(int dd, int len, char *name, int to);
 int hci_write_local_name(int dd, const char *name, int to);
 int hci_read_remote_name(int dd, const bdaddr_t *bdaddr, int len, char *name, int to);
-int hci_read_remote_features(int dd, uint16_t handle, uint8_t *features, int to);
+int hci_read_remote_name_with_clock_offset(int dd, const bdaddr_t *bdaddr, uint8_t pscan_rep_mode, uint16_t clkoffset, int len, char *name, int to);
+int hci_read_remote_name_cancel(int dd, const bdaddr_t *bdaddr, int to);
 int hci_read_remote_version(int dd, uint16_t handle, struct hci_version *ver, int to);
+int hci_read_remote_features(int dd, uint16_t handle, uint8_t *features, int to);
+int hci_read_remote_ext_features(int dd, uint16_t handle, uint8_t page, uint8_t *max_page, uint8_t *features, int to);
 int hci_read_clock_offset(int dd, uint16_t handle, uint16_t *clkoffset, int to);
 int hci_read_local_version(int dd, struct hci_version *ver, int to);
+int hci_read_local_commands(int dd, uint8_t *commands, int to);
 int hci_read_local_features(int dd, uint8_t *features, int to);
+int hci_read_local_ext_features(int dd, uint8_t page, uint8_t *max_page, uint8_t *features, int to);
 int hci_read_bd_addr(int dd, bdaddr_t *bdaddr, int to);
 int hci_read_class_of_dev(int dd, uint8_t *cls, int to);
 int hci_write_class_of_dev(int dd, uint32_t cls, int to);
@@ -81,17 +86,24 @@ int hci_read_voice_setting(int dd, uint16_t *vs, int to);
 int hci_write_voice_setting(int dd, uint16_t vs, int to);
 int hci_read_current_iac_lap(int dd, uint8_t *num_iac, uint8_t *lap, int to);
 int hci_write_current_iac_lap(int dd, uint8_t num_iac, uint8_t *lap, int to);
+int hci_read_stored_link_key(int dd, bdaddr_t *bdaddr, uint8_t all, int to);
+int hci_write_stored_link_key(int dd, bdaddr_t *bdaddr, uint8_t *key, int to);
+int hci_delete_stored_link_key(int dd, bdaddr_t *bdaddr, uint8_t all, int to);
 int hci_authenticate_link(int dd, uint16_t handle, int to);
 int hci_encrypt_link(int dd, uint16_t handle, uint8_t encrypt, int to);
 int hci_change_link_key(int dd, uint16_t handle, int to);
 int hci_switch_role(int dd, bdaddr_t *bdaddr, uint8_t role, int to);
 int hci_park_mode(int dd, uint16_t handle, uint16_t max_interval, uint16_t min_interval, int to);
 int hci_exit_park_mode(int dd, uint16_t handle, int to);
+int hci_read_inquiry_scan_type(int dd, uint8_t *type, int to);
+int hci_write_inquiry_scan_type(int dd, uint8_t type, int to);
 int hci_read_inquiry_mode(int dd, uint8_t *mode, int to);
 int hci_write_inquiry_mode(int dd, uint8_t mode, int to);
 int hci_read_afh_mode(int dd, uint8_t *mode, int to);
 int hci_write_afh_mode(int dd, uint8_t mode, int to);
-int hci_read_transmit_power_level(int dd, uint16_t handle, uint8_t type, uint8_t *level, int to);
+int hci_read_ext_inquiry_response(int dd, uint8_t *fec, uint8_t *data, int to);
+int hci_write_ext_inquiry_response(int dd, uint8_t fec, uint8_t *data, int to);
+int hci_read_transmit_power_level(int dd, uint16_t handle, uint8_t type, int8_t *level, int to);
 int hci_read_link_supervision_timeout(int dd, uint16_t handle, uint16_t *timeout, int to);
 int hci_write_link_supervision_timeout(int dd, uint16_t handle, uint16_t timeout, int to);
 int hci_set_afh_classification(int dd, uint8_t *map, int to);
@@ -103,7 +115,7 @@ int hci_read_clock(int dd, uint16_t handle, uint8_t which, uint32_t *clock, uint
 int hci_local_name(int dd, int len, char *name, int to);
 int hci_remote_name(int dd, const bdaddr_t *bdaddr, int len, char *name, int to);
 
-int hci_for_each_dev(int flag, int(*func)(int s, int dev_id, long arg), long arg);
+int hci_for_each_dev(int flag, int(*func)(int dd, int dev_id, long arg), long arg);
 int hci_get_route(bdaddr_t *bdaddr);
 
 char *hci_dtypetostr(int type);
@@ -117,6 +129,9 @@ int hci_strtolp(char *str, unsigned int *val);
 char *hci_lmtostr(unsigned int ptype);
 int hci_strtolm(char *str, unsigned int *val);
 
+char *hci_cmdtostr(unsigned int cmd);
+char *hci_commandstostr(uint8_t *commands, char *pref, int width);
+
 char *hci_vertostr(unsigned int ver);
 int hci_strtover(char *str, unsigned int *ver);
 char *lmp_vertostr(unsigned int ver);
