@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: bluetooth.c,v 1.26 2004/11/09 00:05:03 holtmann Exp $
+ *  $Id: bluetooth.c,v 1.33 2005/09/04 22:01:16 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,9 +33,11 @@
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <errno.h>
+#include <stdarg.h>
+#include <stdlib.h>
+#include <malloc.h>
+#include <string.h>
 #include <sys/socket.h>
 
 #include <bluetooth/bluetooth.h>
@@ -52,7 +54,7 @@ void baswap(bdaddr_t *dst, const bdaddr_t *src)
 
 char *batostr(const bdaddr_t *ba)
 {
-	char *str = malloc(18);
+	char *str = bt_malloc(18);
 	if (!str)
 		return NULL;
 
@@ -67,7 +69,7 @@ bdaddr_t *strtoba(const char *str)
 	const char *ptr = str;
 	int i;
 
-	uint8_t *ba = malloc(sizeof(bdaddr_t));
+	uint8_t *ba = bt_malloc(sizeof(bdaddr_t));
 	if (!ba)
 		return NULL;
 
@@ -105,6 +107,72 @@ int str2ba(const char *str, bdaddr_t *ba)
 	return 0;
 }
 
+int ba2oui(const bdaddr_t *ba, char *str)
+{
+	uint8_t b[6];
+
+	baswap((bdaddr_t *) b, ba);
+	return sprintf(str, "%2.2X-%2.2X-%2.2X", b[0], b[1], b[2]);
+}
+
+int baprintf(const char *format, ...)
+{
+	va_list ap;
+	int len;
+
+	va_start(ap, format);
+	len = vprintf(format, ap);
+	va_end(ap);
+
+	return len;
+}
+
+int bafprintf(FILE *stream, const char *format, ...)
+{
+	va_list ap;
+	int len;
+
+	va_start(ap, format);
+	len = vfprintf(stream, format, ap);
+	va_end(ap);
+
+	return len;
+}
+
+int basprintf(char *str, const char *format, ...)
+{
+	va_list ap;
+	int len;
+
+	va_start(ap, format);
+	len = vsnprintf(str, (~0U) >> 1, format, ap);
+	va_end(ap);
+
+	return len;
+}
+
+int basnprintf(char *str, size_t size, const char *format, ...)
+{
+	va_list ap;
+	int len;
+
+	va_start(ap, format);
+	len = vsnprintf(str, size, format, ap);
+	va_end(ap);
+
+	return len;
+}
+
+void *bt_malloc(size_t size)
+{
+	return malloc(size);
+}
+
+void bt_free(void *ptr)
+{
+	free(ptr);
+}
+
 /* Bluetooth error codes to Unix errno mapping */
 int bt_error(uint16_t code)
 {
@@ -307,6 +375,12 @@ char *bt_compidtostr(int compid)
 		return "Research In Motion";
 	case 61:
 		return "IPextreme, Inc.";
+	case 62:
+		return "Systems and Chips, Inc";
+	case 63:
+		return "Bluetooth SIG, Inc";
+	case 64:
+		return "Seiko Epson Corporation";
 	case 65535:
 		return "internal use";
 	default:
