@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hci.c,v 1.62 2005/01/29 23:30:04 holtmann Exp $
+ *  $Id: hci.c,v 1.80 2005/09/07 16:14:55 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,18 +33,14 @@
 #endif
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <signal.h>
-#include <fcntl.h>
-#include <syslog.h>
 #include <errno.h>
-#include <termios.h>
 #include <fcntl.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <malloc.h>
+#include <string.h>
 
 #include <sys/param.h>
-#include <sys/uio.h>
 #include <sys/poll.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -102,7 +98,7 @@ static int hci_str2bit(hci_map *map, char *str, unsigned int *val)
 
 static char *hci_uint2str(hci_map *m, unsigned int val) 
 {
-	char *str = malloc(50);
+	char *str = bt_malloc(50);
 	char *ptr = str;
 
 	if (!str)
@@ -313,6 +309,195 @@ int hci_strtolm(char *str, unsigned int *val)
 	return hci_str2bit(link_mode_map, str, val);
 }
 
+/* Command mapping */
+static hci_map commands_map[] = {
+	{ "Inquiry",					0   },
+	{ "Inquiry Cancel",				1   },
+	{ "Periodic Inquiry Mode",			2   },
+	{ "Exit Periodic Inquiry Mode",			3   },
+	{ "Create Connection",				4   },
+	{ "Disconnect",					5   },
+	{ "Add SCO Connection",				6   },
+	{ "Cancel Create Connection",			7   },
+	{ "Accept Connection Request",			8   },
+	{ "Reject Connection Request",			9   },
+	{ "Link Key Request Reply",			10  },
+	{ "Link Key Request Negative Reply",		11  },
+	{ "PIN Code Request Reply",			12  },
+	{ "PIN Code Request Negative Reply",		13  },
+	{ "Change Connection Packet Type",		14  },
+	{ "Authentication Requested",			15  },
+	{ "Set Connection Encryption",			16  },
+	{ "Change Connection Link Key",			17  },
+	{ "Master Link Key",				18  },
+	{ "Remote Name Request",			19  },
+	{ "Cancel Remote Name Request",			20  },
+	{ "Read Remote Supported Features",		21  },
+	{ "Read Remote Extended Features",		22  },
+	{ "Read Remote Version Information",		23  },
+	{ "Read Clock Offset",				24  },
+	{ "Read LMP Handle",				25  },
+	{ "Reserved",					26  },
+	{ "Reserved",					27  },
+	{ "Reserved",					28  },
+	{ "Reserved",					29  },
+	{ "Reserved",					30  },
+	{ "Reserved",					31  },
+	{ "Reserved",					32  },
+	{ "Hold Mode",					33  },
+	{ "Sniff Mode",					34  },
+	{ "Exit Sniff Mode",				35  },
+	{ "Park State",					36  },
+	{ "Exit Park State",				37  },
+	{ "QoS Setup",					38  },
+	{ "Role Discovery",				39  },
+	{ "Switch Role",				40  },
+	{ "Read Link Policy Settings",			41  },
+	{ "Write Link Policy Settings",			42  },
+	{ "Read Default Link Policy Settings",		43  },
+	{ "Write Default Link Policy Settings",		44  },
+	{ "Flow Specification",				45  },
+	{ "Set Event Mask",				46  },
+	{ "Reset",					47  },
+	{ "Set Event Filter",				48  },
+	{ "Flush",					49  },
+	{ "Read PIN Type",				50  },
+	{ "Write PIN Type",				51  },
+	{ "Create New Unit Key",			52  },
+	{ "Read Stored Link Key",			53  },
+	{ "Write Stored Link Key",			54  },
+	{ "Delete Stored Link Key",			55  },
+	{ "Write Local Name",				56  },
+	{ "Read Local Name",				57  },
+	{ "Read Connection Accept Timeout",		58  },
+	{ "Write Connection Accept Timeout",		59  },
+	{ "Read Page Timeout",				60  },
+	{ "Write Page Timeout",				61  },
+	{ "Read Scan Enable",				62  },
+	{ "Write Scan Enable",				63  },
+	{ "Read Page Scan Activity",			64  },
+	{ "Write Page Scan Activity",			65  },
+	{ "Read Inquiry Scan Activity",			66  },
+	{ "Write Inquiry Scan Activity",		67  },
+	{ "Read Authentication Enable",			68  },
+	{ "Write Authentication Enable",		69  },
+	{ "Read Encryption Mode",			70  },
+	{ "Write Encryption Mode",			71  },
+	{ "Read Class Of Device",			72  },
+	{ "Write Class Of Device",			73  },
+	{ "Read Voice Setting",				74  },
+	{ "Write Voice Setting",			75  },
+	{ "Read Automatic Flush Timeout",		76  },
+	{ "Write Automatic Flush Timeout",		77  },
+	{ "Read Num Broadcast Retransmissions",		78  },
+	{ "Write Num Broadcast Retransmissions",	79  },
+	{ "Read Hold Mode Activity",			80  },
+	{ "Write Hold Mode Activity",			81  },
+	{ "Read Transmit Power Level",			82  },
+	{ "Read Synchronous Flow Control Enable",	83  },
+	{ "Write Synchronous Flow Control Enable",	84  },
+	{ "Set Host Controller To Host Flow Control",	85  },
+	{ "Host Buffer Size",				86  },
+	{ "Host Number Of Completed Packets",		87  },
+	{ "Read Link Supervision Timeout",		88  },
+	{ "Write Link Supervision Timeout",		89  },
+	{ "Read Number of Supported IAC",		90  },
+	{ "Read Current IAC LAP",			91  },
+	{ "Write Current IAC LAP",			92  },
+	{ "Reserved",					93  },
+	{ "Reserved",					94  },
+	{ "Read Page Scan Mode",			95  },
+	{ "Write Page Scan Mode",			96  },
+	{ "Set AFH Channel Classification",		97  },
+	{ "reserved",					98  },
+	{ "reserved",					99  },
+	{ "Read Inquiry Scan Type",			100 },
+	{ "Write Inquiry Scan Type",			101 },
+	{ "Read Inquiry Mode",				102 },
+	{ "Write Inquiry Mode",				103 },
+	{ "Read Page Scan Type",			104 },
+	{ "Write Page Scan Type",			105 },
+	{ "Read AFH Channel Assessment Mode",		106 },
+	{ "Write AFH Channel Assessment Mode",		107 },
+	{ "Reserved",					108 },
+	{ "Reserved",					109 },
+	{ "Reserved",					110 },
+	{ "Reserved",					111 },
+	{ "Reserved",					112 },
+	{ "Reserved",					113 },
+	{ "Reserved",					114 },
+	{ "Read Loacal Version Information",		115 },
+	{ "Reserved",					116 },
+	{ "Read Local Supported Features",		117 },
+	{ "Read Local Extended Features",		118 },
+	{ "Read Buffer Sizer",				119 },
+	{ "Read Country Code",				120 },
+	{ "Read BD ADDR",				121 },
+	{ "Read Failed Contact Counter",		122 },
+	{ "Reset Failed Contact Counter",		123 },
+	{ "Get Link Quality",				124 },
+	{ "Read RSSI",					125 },
+	{ "Read AFH Channel Map",			126 },
+	{ "Read BD Clock",				127 },
+	{ "Read Loopback Mode",				128 },
+	{ "Write Loopback Mode",			129 },
+	{ "Enable Device Under Test Mode",		130 },
+	{ "Setup Synchronous Connection",		131 },
+	{ "Accept Synchronous Connection",		132 },
+	{ "Reject Synchronous Connection",		133 },
+	{ "Reserved",					134 },
+	{ "Reserved",					135 },
+	{ "Read Extended Inquiry Response",		136 },
+	{ "Write Extended Inquiry Response",		137 },
+	{ NULL }
+};
+
+char *hci_cmdtostr(unsigned int cmd)
+{
+	return hci_uint2str(commands_map, cmd);
+}
+
+char *hci_commandstostr(uint8_t *commands, char *pref, int width)
+{
+	hci_map *m;
+	char *off, *ptr, *str;
+	int size = 10;
+
+	m = commands_map;
+
+	while (m->str) {
+		if (hci_test_bit(m->val, commands))
+			size += strlen(m->str) + (pref ? strlen(pref) : 0) + 3;
+		m++;
+	}
+
+	str = bt_malloc(size);
+	if (!str)
+		return NULL;
+
+	ptr = str; *ptr = '\0';
+
+	if (pref)
+		ptr += sprintf(ptr, "%s", pref);
+
+	off = ptr;
+
+	m = commands_map;
+
+	while (m->str) {
+		if (hci_test_bit(m->val, commands)) {
+			if (strlen(off) + strlen(m->str) > width - 3) {
+				ptr += sprintf(ptr, "\n%s", pref ? pref : "");
+				off = ptr;
+			}
+			ptr += sprintf(ptr, "'%s' ", m->str);
+		}
+		m++;
+	}
+
+	return str;
+}
+
 /* Version mapping */
 static hci_map ver_map[] = {
 	{ "1.0b",	0x00 },
@@ -324,8 +509,7 @@ static hci_map ver_map[] = {
 
 char *hci_vertostr(unsigned int ver)
 {
-	char *str = hci_uint2str(ver_map, ver);
-	return *str ? str : "n/a";
+	return hci_uint2str(ver_map, ver);
 }
 
 int hci_strtover(char *str, unsigned int *ver)
@@ -335,8 +519,7 @@ int hci_strtover(char *str, unsigned int *ver)
 
 char *lmp_vertostr(unsigned int ver)
 {
-	char *str = hci_uint2str(ver_map, ver);
-	return *str ? str : "n/a";
+	return hci_uint2str(ver_map, ver);
 }
 
 int lmp_strtover(char *str, unsigned int *ver)
@@ -404,6 +587,7 @@ static hci_map lmp_features_map[8][9] = {
 		{ NULL }
 	},
 	{	/* Byte 6 */
+		{ "<extended inquiry>",	LMP_EXT_INQ	},	/* Bit 0 */
 		{ NULL }
 	},
 	{	/* Byte 7 */
@@ -414,25 +598,35 @@ static hci_map lmp_features_map[8][9] = {
 
 char *lmp_featurestostr(uint8_t *features, char *pref, int width)
 {
-	char *off, *ptr, *str = malloc(400);
-	int i;
+	char *off, *ptr, *str;
+	int i, size = 10;
 
+	for (i = 0; i < 8; i++) {
+		hci_map *m = lmp_features_map[i];
+
+		while (m->str) {
+			if (m->val & features[i])
+				size += strlen(m->str) + (pref ? strlen(pref) : 0) + 1;
+			m++;
+		}
+	}
+
+	str = bt_malloc(size);
 	if (!str)
 		return NULL;
 
-	ptr = str; *ptr = 0;
+	ptr = str; *ptr = '\0';
 
 	if (pref)
 		ptr += sprintf(ptr, "%s", pref);
 
 	off = ptr;
 
-	for (i =  0;  i < 8; i++) {
-		hci_map *m;
+	for (i = 0; i < 8; i++) {
+		hci_map *m = lmp_features_map[i];
 
-		m = lmp_features_map[i];
 		while (m->str) {
-			if ((unsigned int) m->val & (unsigned int) features[i]) {
+			if (m->val & features[i]) {
 				if (strlen(off) + strlen(m->str) > width - 1) {
 					ptr += sprintf(ptr, "\n%s", pref ? pref : "");
 					off = ptr;
@@ -448,28 +642,26 @@ char *lmp_featurestostr(uint8_t *features, char *pref, int width)
 
 /* HCI functions that do not require open device */
 
-int hci_for_each_dev(int flag, int (*func)(int s, int dev_id, long arg), long arg)
+int hci_for_each_dev(int flag, int (*func)(int dd, int dev_id, long arg), long arg)
 {
 	struct hci_dev_list_req *dl;
 	struct hci_dev_req *dr;
 	int dev_id = -1;
-	int i, sk;
+	int i, sk, err;
 
 	sk = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
 	if (sk < 0)
 		return -1;
 
 	dl = malloc(HCI_MAX_DEV * sizeof(*dr) + sizeof(*dl));
-	if (!dl) {
-		close(sk);
-		return -1;
-	}
+	if (!dl)
+		goto done;
 
 	dl->dev_num = HCI_MAX_DEV;
 	dr = dl->dev_req;
 
-	if (ioctl(sk, HCIGETDEVLIST, (void *) dl))
-		goto done;
+	if (ioctl(sk, HCIGETDEVLIST, (void *) dl) < 0)
+		goto free;
 
 	for (i = 0; i < dl->dev_num; i++, dr++) {
 		if (hci_test_bit(flag, &dr->dev_opt))
@@ -479,17 +671,25 @@ int hci_for_each_dev(int flag, int (*func)(int s, int dev_id, long arg), long ar
 			}
 	}
 
-done:
-	close(sk);
+	if (dev_id < 0)
+		errno = ENODEV;
+
+free:
 	free(dl);
+
+done:
+	err = errno;
+	close(sk);
+	errno = err;
+
 	return dev_id;
 }
 
-static int __other_bdaddr(int sk, int dev_id, long arg)
+static int __other_bdaddr(int dd, int dev_id, long arg)
 {
 	struct hci_dev_info di = { dev_id: dev_id };
 
-	if (ioctl(sk, HCIGETDEVINFO, (void *) &di))
+	if (ioctl(dd, HCIGETDEVINFO, (void *) &di))
 		return 0;
 
 	if (hci_test_bit(HCI_RAW, &di.flags))
@@ -498,11 +698,11 @@ static int __other_bdaddr(int sk, int dev_id, long arg)
 	return bacmp((bdaddr_t *) arg, &di.bdaddr);
 }
 
-static int __same_bdaddr(int sk, int dev_id, long arg)
+static int __same_bdaddr(int dd, int dev_id, long arg)
 {
 	struct hci_dev_info di = { dev_id: dev_id };
 
-	if (ioctl(sk, HCIGETDEVINFO, (void *) &di))
+	if (ioctl(dd, HCIGETDEVINFO, (void *) &di))
 		return 0;
 
 	return !bacmp((bdaddr_t *) arg, &di.bdaddr);
@@ -510,10 +710,8 @@ static int __same_bdaddr(int sk, int dev_id, long arg)
 
 int hci_get_route(bdaddr_t *bdaddr)
 {
-	if (bdaddr)
-		return hci_for_each_dev(HCI_UP, __other_bdaddr, (long) bdaddr);
-	else
-		return hci_for_each_dev(HCI_UP, NULL, 0);
+	return hci_for_each_dev(HCI_UP, __other_bdaddr,
+				(long) (bdaddr ? bdaddr : BDADDR_ANY));
 }
 
 int hci_devid(const char *str)
@@ -530,22 +728,26 @@ int hci_devid(const char *str)
 		str2ba(str, &ba);
 		id = hci_for_each_dev(HCI_UP, __same_bdaddr, (long) &ba);
 	}
+
 	return id;
 }
 
 int hci_devinfo(int dev_id, struct hci_dev_info *di)
 {
-	int s, err;
+	int dd, err, ret;
 
-	s = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
-	if (s < 0)
-		return s;
+	dd = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
+	if (dd < 0)
+		return dd;
 
 	di->dev_id = dev_id;
-	err = ioctl(s, HCIGETDEVINFO, (void *) di);
-	close(s);
+	ret = ioctl(dd, HCIGETDEVINFO, (void *) di);
 
-	return err;
+	err = errno;
+	close(dd);
+	errno = err;
+
+	return ret;
 }
 
 int hci_devba(int dev_id, bdaddr_t *bdaddr)
@@ -561,36 +763,41 @@ int hci_devba(int dev_id, bdaddr_t *bdaddr)
 	}
 
 	bacpy(bdaddr, &di.bdaddr);
+
 	return 0;
 }
 
 int hci_inquiry(int dev_id, int len, int nrsp, const uint8_t *lap, inquiry_info **ii, long flags)
 {
 	struct hci_inquiry_req *ir;
+	uint8_t num_rsp = nrsp;
 	void *buf;
-	int s, err;
+	int dd, size, err, ret = -1;
 
-	if (nrsp <= 0)
-		nrsp = 200;	/* enough ? */
+	if (nrsp <= 0) {
+		num_rsp = 0;
+		nrsp = 255;
+	}
 
-	if (dev_id < 0 && (dev_id = hci_get_route(NULL)) < 0) {
-		errno = ENODEV;
-		return -1;
+	if (dev_id < 0) {
+		dev_id = hci_get_route(NULL);
+		if (dev_id < 0) {
+			errno = ENODEV;
+			return -1;
+		}
 	}	
 
-	s = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
-	if (s < 0)
-		return -1;
+	dd = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
+	if (dd < 0)
+		return dd;
 
 	buf = malloc(sizeof(*ir) + (sizeof(inquiry_info) * (nrsp)));
-	if (!buf) {
-		close(s);
-		return -1;
-	}
+	if (!buf)
+		goto done;
 
 	ir = buf;
 	ir->dev_id  = dev_id;
-	ir->num_rsp = nrsp;
+	ir->num_rsp = num_rsp;
 	ir->length  = len;
 	ir->flags   = flags;
 
@@ -602,23 +809,30 @@ int hci_inquiry(int dev_id, int len, int nrsp, const uint8_t *lap, inquiry_info
 		ir->lap[2] = 0x9e;
 	}
 
-	err = ioctl(s, HCIINQUIRY, (unsigned long) buf);
-	close(s);
+	ret = ioctl(dd, HCIINQUIRY, (unsigned long) buf);
+	if (ret < 0)
+		goto free;
 
-	if (!err) {
-		int size = sizeof(inquiry_info) * ir->num_rsp;
+	size = sizeof(inquiry_info) * ir->num_rsp;
 
-		if (!*ii) 
-			*ii = (void *) malloc(size);
+	if (!*ii)
+		*ii = malloc(size);
 
-		if (*ii) {
-			memcpy((void *) *ii, buf + sizeof(*ir), size);
-			err = ir->num_rsp;
-		} else
-			err = -1;
-	}
+	if (*ii) {
+		memcpy((void *) *ii, buf + sizeof(*ir), size);
+		ret = ir->num_rsp;
+	} else
+		ret = -1;
+
+free:
 	free(buf);
-	return err;
+
+done:
+	err = errno;
+	close(dd);
+	errno = err;
+
+	return ret;
 }
 
 /* Open HCI device. 
@@ -645,6 +859,7 @@ failed:
 	err = errno;
 	close(dd);
 	errno = err;
+
 	return -1;
 }
 
@@ -691,8 +906,9 @@ int hci_send_req(int dd, struct hci_request *r, int to)
 	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr;
 	uint16_t opcode = htobs(cmd_opcode_pack(r->ogf, r->ocf));
 	struct hci_filter nf, of;
+	socklen_t len;
 	hci_event_hdr *hdr;
-	int err, len, try;
+	int err, try;
 
 	len = sizeof(of);
 	if (getsockopt(dd, SOL_HCI, HCI_FILTER, &of, &len) < 0)
@@ -877,7 +1093,7 @@ int hci_read_local_name(int dd, int len, char *name, int to)
 	}
 
 	rp.name[247] = '\0';
-	strncpy(name, rp.name, len);
+	strncpy(name, (char *) rp.name, len);
 	return 0;
 }
 
@@ -886,7 +1102,9 @@ int hci_write_local_name(int dd, const char *name, int to)
 	change_local_name_cp cp;
 	struct hci_request rq;
 
-	strncpy(cp.name, name, sizeof(cp.name));
+	memset(&cp, 0, sizeof(cp));
+	strncpy((char *) cp.name, name, sizeof(cp.name));
+
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_HOST_CTL;
 	rq.ocf    = OCF_CHANGE_LOCAL_NAME;
@@ -895,10 +1113,11 @@ int hci_write_local_name(int dd, const char *name, int to)
 
 	if (hci_send_req(dd, &rq, to) < 0)
 		return -1;
+
 	return 0;
 }
 
-int hci_read_remote_name(int dd, const bdaddr_t *bdaddr, int len, char *name, int to)
+int hci_read_remote_name_with_clock_offset(int dd, const bdaddr_t *bdaddr, uint8_t pscan_rep_mode, uint16_t clkoffset, int len, char *name, int to)
 {
 	evt_remote_name_req_complete rn;
 	remote_name_req_cp cp;
@@ -906,7 +1125,8 @@ int hci_read_remote_name(int dd, const bdaddr_t *bdaddr, int len, char *name, in
 
 	memset(&cp, 0, sizeof(cp));
 	bacpy(&cp.bdaddr, bdaddr);
-	cp.pscan_rep_mode = 0x02;
+	cp.pscan_rep_mode = pscan_rep_mode;
+	cp.clock_offset   = clkoffset;
 
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_LINK_CTL;
@@ -926,7 +1146,64 @@ int hci_read_remote_name(int dd, const bdaddr_t *bdaddr, int len, char *name, in
 	}
 
 	rn.name[247] = '\0';
-	strncpy(name, rn.name, len);
+	strncpy(name, (char *) rn.name, len);
+	return 0;
+}
+
+int hci_read_remote_name(int dd, const bdaddr_t *bdaddr, int len, char *name, int to)
+{
+	return hci_read_remote_name_with_clock_offset(dd, bdaddr, 0x02, 0x0000, len, name, to);
+}
+
+int hci_read_remote_name_cancel(int dd, const bdaddr_t *bdaddr, int to)
+{
+	remote_name_req_cancel_cp cp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	bacpy(&cp.bdaddr, bdaddr);
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_LINK_CTL;
+	rq.ocf    = OCF_REMOTE_NAME_REQ_CANCEL;
+	rq.cparam = &cp;
+	rq.clen   = REMOTE_NAME_REQ_CANCEL_CP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	return 0;
+}
+
+int hci_read_remote_version(int dd, uint16_t handle, struct hci_version *ver, int to)
+{
+	evt_read_remote_version_complete rp;
+	read_remote_version_cp cp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	cp.handle = handle;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_LINK_CTL;
+	rq.ocf    = OCF_READ_REMOTE_VERSION;
+	rq.event  = EVT_READ_REMOTE_VERSION_COMPLETE;
+	rq.cparam = &cp;
+	rq.clen   = READ_REMOTE_VERSION_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = EVT_READ_REMOTE_VERSION_COMPLETE_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	ver->manufacturer = btohs(rp.manufacturer);
+	ver->lmp_ver      = rp.lmp_ver;
+	ver->lmp_subver   = btohs(rp.lmp_subver);
 	return 0;
 }
 
@@ -960,23 +1237,24 @@ int hci_read_remote_features(int dd, uint16_t handle, uint8_t *features, int to)
 	return 0;
 }
 
-int hci_read_remote_version(int dd, uint16_t handle, struct hci_version *ver, int to)
+int hci_read_remote_ext_features(int dd, uint16_t handle, uint8_t page, uint8_t *max_page, uint8_t *features, int to)
 {
-	evt_read_remote_version_complete rp;
-	read_remote_version_cp cp;
+	evt_read_remote_ext_features_complete rp;
+	read_remote_ext_features_cp cp;
 	struct hci_request rq;
 
 	memset(&cp, 0, sizeof(cp));
-	cp.handle = handle;
+	cp.handle   = handle;
+	cp.page_num = page;
 
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_LINK_CTL;
-	rq.ocf    = OCF_READ_REMOTE_VERSION;
-	rq.event  = EVT_READ_REMOTE_VERSION_COMPLETE;
+	rq.ocf    = OCF_READ_REMOTE_EXT_FEATURES;
+	rq.event  = EVT_READ_REMOTE_EXT_FEATURES_COMPLETE;
 	rq.cparam = &cp;
-	rq.clen   = READ_REMOTE_VERSION_CP_SIZE;
+	rq.clen   = READ_REMOTE_EXT_FEATURES_CP_SIZE;
 	rq.rparam = &rp;
-	rq.rlen   = EVT_READ_REMOTE_VERSION_COMPLETE_SIZE;
+	rq.rlen   = EVT_READ_REMOTE_EXT_FEATURES_COMPLETE_SIZE;
 
 	if (hci_send_req(dd, &rq, to) < 0)
 		return -1;
@@ -986,9 +1264,8 @@ int hci_read_remote_version(int dd, uint16_t handle, struct hci_version *ver, in
 		return -1;
 	}
 
-	ver->manufacturer = btohs(rp.manufacturer);
-	ver->lmp_ver      = rp.lmp_ver;
-	ver->lmp_subver   = btohs(rp.lmp_subver);
+	*max_page = rp.max_page_num;
+	memcpy(features, rp.features, 8);
 	return 0;
 }
 
@@ -1049,6 +1326,29 @@ int hci_read_local_version(int dd, struct hci_version *ver, int to)
 	return 0;
 }
 
+int hci_read_local_commands(int dd, uint8_t *commands, int to)
+{
+	read_local_commands_rp rp;
+	struct hci_request rq;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_INFO_PARAM;
+	rq.ocf    = OCF_READ_LOCAL_COMMANDS;
+	rq.rparam = &rp;
+	rq.rlen   = READ_LOCAL_COMMANDS_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	memcpy(commands, rp.commands, 64);
+	return 0;
+}
+
 int hci_read_local_features(int dd, uint8_t *features, int to)
 {
 	read_local_features_rp rp;
@@ -1072,6 +1372,35 @@ int hci_read_local_features(int dd, uint8_t *features, int to)
 	return 0;
 }
 
+int hci_read_local_ext_features(int dd, uint8_t page, uint8_t *max_page, uint8_t *features, int to)
+{
+	read_local_ext_features_cp cp;
+	read_local_ext_features_rp rp;
+	struct hci_request rq;
+
+	cp.page_num = page;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_INFO_PARAM;
+	rq.ocf    = OCF_READ_LOCAL_EXT_FEATURES;
+	rq.cparam = &cp;
+	rq.clen   = READ_LOCAL_EXT_FEATURES_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = READ_LOCAL_EXT_FEATURES_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	*max_page = rp.max_page_num;
+	memcpy(features, rp.features, 8);
+	return 0;
+}
+
 int hci_read_bd_addr(int dd, bdaddr_t *bdaddr, int to)
 {
 	read_bd_addr_rp rp;
@@ -1209,7 +1538,62 @@ int hci_write_current_iac_lap(int dd, uint8_t num_iac, uint8_t *lap, int to)
 	rq.ogf    = OGF_HOST_CTL;
 	rq.ocf    = OCF_WRITE_CURRENT_IAC_LAP;
 	rq.cparam = &cp;
-	rq.clen   = WRITE_CURRENT_IAC_LAP_CP_SIZE;
+	rq.clen   = num_iac * 3 + 1;
+
+	return hci_send_req(dd, &rq, to);
+}
+
+int hci_read_stored_link_key(int dd, bdaddr_t *bdaddr, uint8_t all, int to)
+{
+	read_stored_link_key_cp cp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	bacpy(&cp.bdaddr, bdaddr);
+	cp.read_all = all;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_READ_STORED_LINK_KEY;
+	rq.cparam = &cp;
+	rq.clen   = READ_STORED_LINK_KEY_CP_SIZE;
+
+	return hci_send_req(dd, &rq, to);
+}
+
+int hci_write_stored_link_key(int dd, bdaddr_t *bdaddr, uint8_t *key, int to)
+{
+	unsigned char cp[WRITE_STORED_LINK_KEY_CP_SIZE + 6 + 16];
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	cp[0] = 1;
+	bacpy((bdaddr_t *) (cp + 1), bdaddr);
+	memcpy(cp + 7, key, 16);
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_WRITE_STORED_LINK_KEY;
+	rq.cparam = &cp;
+	rq.clen   = WRITE_STORED_LINK_KEY_CP_SIZE + 6 + 16;
+
+	return hci_send_req(dd, &rq, to);
+}
+
+int hci_delete_stored_link_key(int dd, bdaddr_t *bdaddr, uint8_t all, int to)
+{
+	delete_stored_link_key_cp cp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	bacpy(&cp.bdaddr, bdaddr);
+	cp.delete_all = all;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_DELETE_STORED_LINK_KEY;
+	rq.cparam = &cp;
+	rq.clen   = DELETE_STORED_LINK_KEY_CP_SIZE;
 
 	return hci_send_req(dd, &rq, to);
 }
@@ -1383,6 +1767,57 @@ int hci_exit_park_mode(int dd, uint16_t handle, int to)
 	return 0;
 }
 
+int hci_read_inquiry_scan_type(int dd, uint8_t *type, int to)
+{
+	read_inquiry_scan_type_rp rp;
+	struct hci_request rq;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_READ_INQUIRY_SCAN_TYPE;
+	rq.rparam = &rp;
+	rq.rlen   = READ_INQUIRY_SCAN_TYPE_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	*type = rp.type;
+	return 0;
+}
+
+int hci_write_inquiry_scan_type(int dd, uint8_t type, int to)
+{
+	write_inquiry_scan_type_cp cp;
+	write_inquiry_scan_type_rp rp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	cp.type = type;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_WRITE_INQUIRY_SCAN_TYPE;
+	rq.cparam = &cp;
+	rq.clen   = WRITE_INQUIRY_SCAN_TYPE_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = WRITE_INQUIRY_SCAN_TYPE_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	return 0;
+}
+
 int hci_read_inquiry_mode(int dd, uint8_t *mode, int to)
 {
 	read_inquiry_mode_rp rp;
@@ -1485,7 +1920,61 @@ int hci_write_afh_mode(int dd, uint8_t mode, int to)
 	return 0;
 }
 
-int hci_read_transmit_power_level(int dd, uint16_t handle, uint8_t type, uint8_t *level, int to)
+int hci_read_ext_inquiry_response(int dd, uint8_t *fec, uint8_t *data, int to)
+{
+	read_ext_inquiry_response_rp rp;
+	struct hci_request rq;
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_READ_EXT_INQUIRY_RESPONSE;
+	rq.rparam = &rp;
+	rq.rlen   = READ_EXT_INQUIRY_RESPONSE_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	*fec = rp.fec;
+	memcpy(data, rp.data, 240);
+
+	return 0;
+}
+
+int hci_write_ext_inquiry_response(int dd, uint8_t fec, uint8_t *data, int to)
+{
+	write_ext_inquiry_response_cp cp;
+	write_ext_inquiry_response_rp rp;
+	struct hci_request rq;
+
+	memset(&cp, 0, sizeof(cp));
+	cp.fec = fec;
+	memcpy(cp.data, data, 240);
+
+	memset(&rq, 0, sizeof(rq));
+	rq.ogf    = OGF_HOST_CTL;
+	rq.ocf    = OCF_WRITE_EXT_INQUIRY_RESPONSE;
+	rq.cparam = &cp;
+	rq.clen   = WRITE_EXT_INQUIRY_RESPONSE_CP_SIZE;
+	rq.rparam = &rp;
+	rq.rlen   = WRITE_EXT_INQUIRY_RESPONSE_RP_SIZE;
+
+	if (hci_send_req(dd, &rq, to) < 0)
+		return -1;
+
+	if (rp.status) {
+		errno = EIO;
+		return -1;
+	}
+
+	return 0;
+}
+
+int hci_read_transmit_power_level(int dd, uint16_t handle, uint8_t type, int8_t *level, int to)
 {
 	read_transmit_power_level_cp cp;
 	read_transmit_power_level_rp rp;
