@@ -26,7 +26,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: sdp.c,v 1.17 2005/01/27 23:14:46 holtmann Exp $
+ *  $Id: sdp.c,v 1.27 2005/08/07 11:19:58 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,6 +41,8 @@
 #include <syslog.h>
 #include <sys/un.h>
 #include <sys/socket.h>
+#include <sys/time.h>
+#include <sys/types.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
@@ -178,6 +180,8 @@ static struct tupla ServiceClass[] = {
 	{ UDI_TA_SVCLASS_ID,			"UDI TA"			},
 	{ AV_SVCLASS_ID,			"Audio/Video"			},
 	{ SAP_SVCLASS_ID,			"SIM Access"			},
+	{ PBAP_PCE_SVCLASS_ID,			"Phonebook Access - PCE"	},
+	{ PBAP_PSE_SVCLASS_ID,			"Phonebook Access - PSE"	},
 	{ PNP_INFO_SVCLASS_ID,			"PnP Information"		},
 	{ GENERIC_NETWORKING_SVCLASS_ID,	"Generic Networking"		},
 	{ GENERIC_FILETRANS_SVCLASS_ID,		"Generic File Transfer"		},
@@ -190,6 +194,7 @@ static struct tupla ServiceClass[] = {
 	{ UPNP_L2CAP_SVCLASS_ID,		"UPnP L2CAP"			},
 	{ VIDEO_SOURCE_SVCLASS_ID,		"Video Source"			},
 	{ VIDEO_SINK_SVCLASS_ID,		"Video Sink"			},
+	{ VIDEO_DISTRIBUTION_SVCLASS_ID,	"Video Distribution"		},
 	{ 0 }
 };
 
@@ -532,7 +537,7 @@ void sdp_attr_remove(sdp_record_t *rec, uint16_t attr)
 		rec->attrlist = sdp_list_remove(rec->attrlist, d);
 }
 
-void sdp_set_seq_len(char *ptr, int length)
+void sdp_set_seq_len(uint8_t *ptr, uint32_t length)
 {
 	uint8_t dtd = *(uint8_t *) ptr++;
 
@@ -612,14 +617,14 @@ static int get_data_size(sdp_buf_t *buf, sdp_data_t *sdpdata)
 
 int sdp_gen_pdu(sdp_buf_t *buf, sdp_data_t *d)
 {
-	int pdu_size = 0, data_size = 0;
+	uint32_t pdu_size = 0, data_size = 0;
 	unsigned char *src = NULL, is_seq = 0, is_alt = 0;
 	uint8_t dtd = d->dtd;
 	uint16_t u16;
 	uint32_t u32;
 	uint64_t u64;
 	uint128_t u128;
-	char *seqp = buf->data + buf->data_size;
+	uint8_t *seqp = buf->data + buf->data_size;
 
 	pdu_size = sdp_set_data_type(buf, dtd);
 	switch (dtd) {
@@ -733,7 +738,7 @@ static void sdp_attr_pdu(void *value, void *udata)
 
 int sdp_gen_record_pdu(const sdp_record_t *rec, sdp_buf_t *buf)
 {
-	buf->data = (char *)malloc(SDP_PDU_CHUNK_SIZE);
+	buf->data = malloc(SDP_PDU_CHUNK_SIZE);
 	if (buf->data) {
 		buf->buf_size = SDP_PDU_CHUNK_SIZE;
 		buf->data_size = 0;
@@ -843,7 +848,7 @@ static sdp_data_t *extract_int(const void *p, int *len)
 	return d;
 }
 
-static sdp_data_t *extract_uuid(const void *p, int *len, sdp_record_t *rec)
+static sdp_data_t *extract_uuid(const uint8_t *p, int *len, sdp_record_t *rec)
 {
 	sdp_data_t *d = (sdp_data_t *) malloc(sizeof(sdp_data_t));
 
@@ -940,7 +945,7 @@ static sdp_data_t *extract_seq(const void *p, int *len, sdp_record_t *rec)
 	return d;
 }
 
-sdp_data_t *sdp_extract_attr(const char *p, int *size, sdp_record_t *rec)
+sdp_data_t *sdp_extract_attr(const uint8_t *p, int *size, sdp_record_t *rec)
 {
 	sdp_data_t *elem;
 	int n = 0;
@@ -1008,17 +1013,19 @@ static void attr_print_func(void *value, void *userData)
 
 void sdp_print_service_attr(sdp_list_t *svcAttrList)
 {
+	SDPDBG("Printing service attr list %p\n", svcAttrList);
 	sdp_list_foreach(svcAttrList, attr_print_func, NULL);
+	SDPDBG("Printed service attr list %p\n", svcAttrList);
 }
 #endif
 
-sdp_record_t *sdp_extract_pdu(const char *buf, int *scanned)
+sdp_record_t *sdp_extract_pdu(const uint8_t *buf, int *scanned)
 {
 	int extracted = 0, seqlen = 0;
 	uint8_t dtd;
 	uint16_t attr;
 	sdp_record_t *rec = sdp_record_alloc();
-	const char *p = buf;
+	const uint8_t *p = buf;
 
 	*scanned = sdp_extract_seqtype(buf, &dtd, &seqlen);
 	p += *scanned;
@@ -1151,7 +1158,7 @@ sdp_data_t *sdp_data_get(const sdp_record_t *rec, uint16_t attrId)
  * Extract the sequence type and its length, and return offset into buf
  * or 0 on failure.
  */
-int sdp_extract_seqtype(const char *buf, uint8_t *dtdp, int *size)
+int sdp_extract_seqtype(const uint8_t *buf, uint8_t *dtdp, int *size)
 {
 	uint8_t dtd = *(uint8_t *) buf;
 	int scanned = sizeof(uint8_t);
@@ -1181,9 +1188,9 @@ int sdp_extract_seqtype(const char *buf, uint8_t *dtdp, int *size)
 	return scanned;
 }
 
-int sdp_send_req(sdp_session_t *session, char *buf, int size)
+int sdp_send_req(sdp_session_t *session, uint8_t *buf, uint32_t size)
 {
-	int sent = 0;
+	uint32_t sent = 0;
 
 	while (sent < size) {
 		int n = send(session->sock, buf + sent, size - sent, 0);
@@ -1194,7 +1201,7 @@ int sdp_send_req(sdp_session_t *session, char *buf, int size)
 	return 0;
 }
 
-int sdp_read_rsp(sdp_session_t *session, char *buf, int size)
+int sdp_read_rsp(sdp_session_t *session, uint8_t *buf, uint32_t size)
 {
 	fd_set readFds;
 	struct timeval timeout = { SDP_RESPONSE_TIMEOUT, 0 };
@@ -1212,7 +1219,7 @@ int sdp_read_rsp(sdp_session_t *session, char *buf, int size)
 /*
  * generic send request, wait for response method.
  */
-int sdp_send_req_w4_rsp(sdp_session_t *session, char *reqbuf, char *rspbuf, int reqsize, int *rspsize)
+int sdp_send_req_w4_rsp(sdp_session_t *session, uint8_t *reqbuf, uint8_t *rspbuf, uint32_t reqsize, uint32_t *rspsize)
 {
 	int n;
 	sdp_pdu_hdr_t *reqhdr = (sdp_pdu_hdr_t *)reqbuf;
@@ -1948,7 +1955,7 @@ uint128_t *sdp_create_base_uuid(void)
 	char *dataPtr;
 	char temp[10];
 	int toBeCopied;
-	char *data;
+	uint8_t *data;
 
 	if (bluetooth_base_uuid == NULL) {
 		strcpy(baseStr, BASE_UUID);
@@ -2178,7 +2185,7 @@ int sdp_uuid_to_proto(uuid_t *uuid)
 	return 0;
 }
 
-int sdp_uuid_extract(const char *p, uuid_t *uuid, int *scanned)
+int sdp_uuid_extract(const uint8_t *p, uuid_t *uuid, int *scanned)
 {
 	uint8_t type = *(const uint8_t *) p;
 
@@ -2210,9 +2217,9 @@ int sdp_uuid_extract(const char *p, uuid_t *uuid, int *scanned)
  * Should the PDU length exceed 2^8, then sequence type is
  * set accordingly and the data is memmove()'d.
  */
-void sdp_append_to_buf(sdp_buf_t *dst, char *data, int len)
+void sdp_append_to_buf(sdp_buf_t *dst, uint8_t *data, uint32_t len)
 {
-	char *p = dst->data;
+	uint8_t *p = dst->data;
 	uint8_t dtd = *(uint8_t *) p;
 
 	SDPDBG("Append src size: %d\n", len);
@@ -2220,7 +2227,7 @@ void sdp_append_to_buf(sdp_buf_t *dst, char *data, int len)
 	SDPDBG("Dst buffer size: %d\n", dst->buf_size);
 	if (dst->data_size + len > dst->buf_size) {
 		int need = SDP_PDU_CHUNK_SIZE * ((len / SDP_PDU_CHUNK_SIZE) + 1);
-		dst->data = (char *)realloc(dst->data, dst->buf_size + need);
+		dst->data = realloc(dst->data, dst->buf_size + need);
 
 		SDPDBG("Realloc'ing : %d\n", need);
 
@@ -2269,7 +2276,7 @@ void sdp_append_to_buf(sdp_buf_t *dst, char *data, int len)
 
 void sdp_append_to_pdu(sdp_buf_t *pdu, sdp_data_t *d)
 {
-	char buf[SDP_SEQ_PDUFORM_SIZE];
+	uint8_t buf[SDP_SEQ_PDUFORM_SIZE];
 	sdp_buf_t append;
 
 	append.data = buf;
@@ -2286,15 +2293,13 @@ void sdp_append_to_pdu(sdp_buf_t *pdu, sdp_data_t *d)
  * It is incorrect to call this method on a record that
  * has been already registered with the server.
  *
- * Returns a non-null value (a pointer) to a service
- * record if successful, else -1 setting errno
+ * Returns zero on success, otherwise -1 (and sets errno).
  */
-int sdp_record_register(sdp_session_t *session, sdp_record_t *rec, uint8_t flags)
+int sdp_device_record_register(sdp_session_t *session, bdaddr_t *device, sdp_record_t *rec, uint8_t flags)
 {
-	char *p;
 	int status = 0;
-	char *req, *rsp;
-	int reqsize, rspsize;
+	uint8_t *req, *rsp, *p;
+	uint32_t reqsize, rspsize;
 	sdp_pdu_hdr_t *reqhdr, *rsphdr;
 	sdp_buf_t pdu;
 
@@ -2304,8 +2309,8 @@ int sdp_record_register(sdp_session_t *session, sdp_record_t *rec, uint8_t flags
 		errno = EREMOTE;
 		return -1;
 	}
-	req = (char *) malloc(SDP_REQ_BUFFER_SIZE);
-	rsp = (char *) malloc(SDP_RSP_BUFFER_SIZE);
+	req = malloc(SDP_REQ_BUFFER_SIZE);
+	rsp = malloc(SDP_RSP_BUFFER_SIZE);
 	if (req == NULL || rsp == NULL) {
 		status = -1;
 		errno = ENOMEM;
@@ -2316,8 +2321,14 @@ int sdp_record_register(sdp_session_t *session, sdp_record_t *rec, uint8_t flags
 	reqhdr->tid    = htons(sdp_gen_tid(session));
 	reqsize = sizeof(sdp_pdu_hdr_t) + 1;
 	p = req + sizeof(sdp_pdu_hdr_t);
-	*p++ = flags;
-	if (0 > sdp_gen_record_pdu(rec, &pdu)) {
+	if (bacmp(device, BDADDR_ANY)) {
+		*p++ = flags | SDP_DEVICE_RECORD;
+		bacpy((bdaddr_t *) p, device);
+		p += sizeof(bdaddr_t);
+		reqsize += sizeof(bdaddr_t);
+	} else
+		*p++ = flags;
+	if (sdp_gen_record_pdu(rec, &pdu) < 0) {
 		status = -1;
 		errno = ENOMEM;
 		goto end;
@@ -2346,15 +2357,19 @@ end:
 	return status;
 }
 
+int sdp_record_register(sdp_session_t *session, sdp_record_t *rec, uint8_t flags)
+{
+	return sdp_device_record_register(session, BDADDR_ANY, rec, flags);
+}
+
 /*
  * unregister a service record
  */
-int sdp_record_unregister(sdp_session_t *session, sdp_record_t *rec)
+int sdp_device_record_unregister(sdp_session_t *session, bdaddr_t *device, sdp_record_t *rec)
 {
-	char *p;
 	int status = 0;
-	char *reqbuf, *rspbuf;
-	int reqsize = 0, rspsize = 0;
+	uint8_t *reqbuf, *rspbuf, *p;
+	uint32_t reqsize = 0, rspsize = 0;
 	sdp_pdu_hdr_t *reqhdr, *rsphdr;
 	uint32_t handle = 0;
 
@@ -2369,8 +2384,8 @@ int sdp_record_unregister(sdp_session_t *session, sdp_record_t *rec)
 		errno = EREMOTE;
 		return -1;
 	}
-	reqbuf = (char *) malloc(SDP_REQ_BUFFER_SIZE);
-	rspbuf = (char *) malloc(SDP_RSP_BUFFER_SIZE);
+	reqbuf = malloc(SDP_REQ_BUFFER_SIZE);
+	rspbuf = malloc(SDP_RSP_BUFFER_SIZE);
 	if (!reqbuf || !rspbuf) {
 		errno = ENOMEM;
 		status = -1;
@@ -2404,15 +2419,19 @@ end:
 	return status;
 }
 
+int sdp_record_unregister(sdp_session_t *session, sdp_record_t *rec)
+{
+	return sdp_device_record_unregister(session, BDADDR_ANY, rec);
+}
+
 /*
  * modify an existing service record
  */
-int sdp_record_update(sdp_session_t *session, const sdp_record_t *rec)
+int sdp_device_record_update(sdp_session_t *session, bdaddr_t *device, const sdp_record_t *rec)
 {
-	char *p;
 	int status = 0;
-	char *reqbuf, *rspbuf;
-	int reqsize, rspsize;
+	uint8_t *reqbuf, *rspbuf, *p;
+	uint32_t reqsize, rspsize;
 	sdp_pdu_hdr_t *reqhdr, *rsphdr;
 	uint32_t handle;
 	sdp_buf_t pdu;
@@ -2428,8 +2447,8 @@ int sdp_record_update(sdp_session_t *session, const sdp_record_t *rec)
 		errno = EREMOTE;
 		return -1;
 	}
-	reqbuf = (char *) malloc(SDP_REQ_BUFFER_SIZE);
-	rspbuf = (char *) malloc(SDP_RSP_BUFFER_SIZE);
+	reqbuf = malloc(SDP_REQ_BUFFER_SIZE);
+	rspbuf = malloc(SDP_RSP_BUFFER_SIZE);
 	if (!reqbuf || !rspbuf) {
 		errno = ENOMEM;
 		status = -1;
@@ -2439,7 +2458,7 @@ int sdp_record_update(sdp_session_t *session, const sdp_record_t *rec)
 	reqhdr->pdu_id = SDP_SVC_UPDATE_REQ;
 	reqhdr->tid    = htons(sdp_gen_tid(session));
 
-	p = (char *) (reqbuf + sizeof(sdp_pdu_hdr_t));
+	p = reqbuf + sizeof(sdp_pdu_hdr_t);
 	reqsize = sizeof(sdp_pdu_hdr_t);
 
 	bt_put_unaligned(htonl(handle), (uint32_t *) p);
@@ -2472,6 +2491,11 @@ end:
 	return status;
 }
 
+int sdp_record_update(sdp_session_t *session, const sdp_record_t *rec)
+{
+	return sdp_device_record_update(session, BDADDR_ANY, rec);
+}
+
 sdp_record_t *sdp_record_alloc()
 {
 	sdp_record_t *rec = (sdp_record_t *)malloc(sizeof(sdp_record_t));
@@ -2520,10 +2544,10 @@ void sdp_pattern_add_uuidseq(sdp_record_t *rec, sdp_list_t *seq)
  * handles are not in "data element sequence" form, but just like
  * an array of service handles
  */
-static void extract_record_handle_seq(char *pdu, sdp_list_t **seq, int count, int *scanned)
+static void extract_record_handle_seq(uint8_t *pdu, sdp_list_t **seq, int count, int *scanned)
 {
 	sdp_list_t *pSeq = *seq;
-	char *pdata = pdu;
+	uint8_t *pdata = pdu;
 	int n;
 
 	for (n = 0; n < count; n++) {
@@ -2539,7 +2563,7 @@ static void extract_record_handle_seq(char *pdu, sdp_list_t **seq, int count, in
  * Generate the attribute sequence pdu form
  * from sdp_list_t elements. Return length of attr seq
  */
-static int gen_dataseq_pdu(char *dst, const sdp_list_t *seq, uint8_t dtd)
+static int gen_dataseq_pdu(uint8_t *dst, const sdp_list_t *seq, uint8_t dtd)
 {
 	sdp_data_t *dataseq;
 	void **types, **values;
@@ -2550,7 +2574,7 @@ static int gen_dataseq_pdu(char *dst, const sdp_list_t *seq, uint8_t dtd)
 	SDPDBG("");
 	
 	memset(&buf, 0, sizeof(sdp_buf_t));
-	buf.data = (char *)malloc(SDP_UUID_SEQ_SIZE);
+	buf.data = malloc(SDP_UUID_SEQ_SIZE);
 	buf.buf_size = SDP_UUID_SEQ_SIZE;
 
 	SDPDBG("Seq length : %d\n", seqlen);
@@ -2580,18 +2604,18 @@ static int gen_dataseq_pdu(char *dst, const sdp_list_t *seq, uint8_t dtd)
 	return seqlen;
 }
 
-static int gen_searchseq_pdu(char *dst, const sdp_list_t *seq)
+static int gen_searchseq_pdu(uint8_t *dst, const sdp_list_t *seq)
 {
-	uuid_t *uuid = (uuid_t *)seq->data;
+	uuid_t *uuid = (uuid_t *) seq->data;
 	return gen_dataseq_pdu(dst, seq, uuid->type);
 }
 
-static int gen_attridseq_pdu(char *dst, const sdp_list_t *seq, uint8_t dataType)
+static int gen_attridseq_pdu(uint8_t *dst, const sdp_list_t *seq, uint8_t dataType)
 {
 	return gen_dataseq_pdu(dst, seq, dataType);
 }
 
-static int copy_cstate(char *pdata, const sdp_cstate_t *cstate)
+static int copy_cstate(uint8_t *pdata, const sdp_cstate_t *cstate)
 {
 	if (cstate) {
 		*pdata++ = cstate->length;
@@ -2629,18 +2653,18 @@ static int copy_cstate(char *pdata, const sdp_cstate_t *cstate)
  *   sdp_list_t **rsp_list
  *     This variable is set on a successful return if there are
  *     non-zero service handles. It is a singly linked list of
- *     service records (sdp_record_t *)
+ *     service record handles (uint16_t)
  */
 int sdp_service_search_req(sdp_session_t *session, const sdp_list_t *search,
 			uint16_t max_rec_num, sdp_list_t **rsp)
 {
 	int status = 0;
-	int reqsize = 0, _reqsize;
-	int rspsize = 0, rsplen;
+	uint32_t reqsize = 0, _reqsize;
+	uint32_t rspsize = 0, rsplen;
 	int seqlen = 0;
 	int scanned, total_rec_count, rec_count;
-	char *pdata, *_pdata;
-	char *reqbuf, *rspbuf;
+	uint8_t *pdata, *_pdata;
+	uint8_t *reqbuf, *rspbuf;
 	sdp_pdu_hdr_t *reqhdr, *rsphdr;
 	sdp_cstate_t *cstate = NULL;
 
@@ -2779,12 +2803,12 @@ sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle,
 			sdp_attrreq_type_t reqtype, const sdp_list_t *attrids)
 {
 	int status = 0;
-	int reqsize = 0, _reqsize;
-	int rspsize = 0, rsp_count;
+	uint32_t reqsize = 0, _reqsize;
+	uint32_t rspsize = 0, rsp_count;
 	int attr_list_len = 0;
 	int seqlen = 0;
-	char *pdata, *_pdata;
-	char *reqbuf, *rspbuf;
+	uint8_t *pdata, *_pdata;
+	uint8_t *reqbuf, *rspbuf;
 	sdp_pdu_hdr_t *reqhdr, *rsphdr;
 	sdp_cstate_t *cstate = NULL;
 	uint8_t cstate_len = 0;
@@ -2796,8 +2820,8 @@ sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle,
 		return 0;
 	}
 
-	reqbuf = (char *) malloc(SDP_REQ_BUFFER_SIZE);
-	rspbuf = (char *) malloc(SDP_RSP_BUFFER_SIZE);
+	reqbuf = malloc(SDP_REQ_BUFFER_SIZE);
+	rspbuf = malloc(SDP_RSP_BUFFER_SIZE);
 	if (!reqbuf || !rspbuf) {
 		errno = ENOMEM;
 		status = -1;
@@ -2871,12 +2895,12 @@ sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle,
 		 * and the last one (which has cstate_len == 0)
 		 */
 		if (cstate_len > 0 || rsp_concat_buf.data_size != 0) {
-			char *targetPtr = NULL;
+			uint8_t *targetPtr = NULL;
 
 			cstate = cstate_len > 0 ? (sdp_cstate_t *) (pdata + rsp_count) : 0;
 
 			// build concatenated response buffer
-			rsp_concat_buf.data = (char *) realloc(rsp_concat_buf.data, rsp_concat_buf.data_size + rsp_count);
+			rsp_concat_buf.data = realloc(rsp_concat_buf.data, rsp_concat_buf.data_size + rsp_count);
 			rsp_concat_buf.buf_size = rsp_concat_buf.data_size + rsp_count;
 			targetPtr = rsp_concat_buf.data + rsp_concat_buf.data_size;
 			memcpy(targetPtr, pdata, rsp_count);
@@ -2948,12 +2972,12 @@ sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle,
 int sdp_service_search_attr_req(sdp_session_t *session, const sdp_list_t *search, sdp_attrreq_type_t reqtype, const sdp_list_t *attrids, sdp_list_t **rsp)
 {
 	int status = 0;
-	int reqsize = 0, _reqsize;
-	int rspsize = 0;
+	uint32_t reqsize = 0, _reqsize;
+	uint32_t rspsize = 0;
 	int seqlen = 0, attr_list_len = 0;
 	int rsp_count = 0, cstate_len = 0;
-	char *pdata, *_pdata;
-	char *reqbuf, *rspbuf;
+	uint8_t *pdata, *_pdata;
+	uint8_t *reqbuf, *rspbuf;
 	sdp_pdu_hdr_t *reqhdr, *rsphdr;
 	uint8_t dataType;
 	sdp_list_t *rec_list = NULL;
@@ -2964,8 +2988,8 @@ int sdp_service_search_attr_req(sdp_session_t *session, const sdp_list_t *search
 		errno = EINVAL;
 		return -1;
 	}
-	reqbuf = (char *) malloc(SDP_REQ_BUFFER_SIZE);
-	rspbuf = (char *) malloc(SDP_RSP_BUFFER_SIZE);
+	reqbuf = malloc(SDP_REQ_BUFFER_SIZE);
+	rspbuf = malloc(SDP_RSP_BUFFER_SIZE);
 	if (!reqbuf || !rspbuf) {
 		errno = ENOMEM;
 		status = -1;
@@ -3045,12 +3069,12 @@ int sdp_service_search_attr_req(sdp_session_t *session, const sdp_list_t *search
 		 * responses and the last one which will have cstate_len == 0
 		 */
 		if (cstate_len > 0 || rsp_concat_buf.data_size != 0) {
-			char *targetPtr = NULL;
+			uint8_t *targetPtr = NULL;
 
 			cstate = cstate_len > 0 ? (sdp_cstate_t *) (pdata + rsp_count) : 0;
 
 			// build concatenated response buffer
-			rsp_concat_buf.data = (char *)realloc(rsp_concat_buf.data, rsp_concat_buf.data_size + rsp_count);
+			rsp_concat_buf.data = realloc(rsp_concat_buf.data, rsp_concat_buf.data_size + rsp_count);
 			targetPtr = rsp_concat_buf.data + rsp_concat_buf.data_size;
 			rsp_concat_buf.buf_size = rsp_concat_buf.data_size + rsp_count;
 			memcpy(targetPtr, pdata, rsp_count);
@@ -3164,13 +3188,17 @@ sdp_session_t *sdp_connect(const bdaddr_t *src, const bdaddr_t *dst, uint32_t fl
 			sa.l2_psm = 0;
 			if (bacmp(src, BDADDR_ANY) != 0) {
 				sa.l2_bdaddr = *src;
-				if (0 > bind(session->sock, (struct sockaddr *)&sa, sizeof(sa)))
+				if (bind(session->sock, (struct sockaddr *) &sa, sizeof(sa)) < 0)
 					goto fail;
 			}
+			if (flags & SDP_WAIT_ON_CLOSE) {
+				struct linger l = { .l_onoff = 1, .l_linger = 1 };
+				setsockopt(session->sock, SOL_SOCKET, SO_LINGER, &l, sizeof(l));
+			}
 			sa.l2_psm = htobs(SDP_PSM);
 			sa.l2_bdaddr = *dst;
 			do
-				if (connect(session->sock, (struct sockaddr *)&sa, sizeof(sa)) == 0)
+				if (connect(session->sock, (struct sockaddr *) &sa, sizeof(sa)) == 0)
 					return session;
 			while (errno == EBUSY && (flags & SDP_RETRY_IF_BUSY));
 		}
